@@ -7,57 +7,43 @@ Description: Module containing the ATOM class.
 
 """
 
-# Standard packages
 import tempfile
-import numpy as np
-import pandas as pd
-from scipy import stats
 from copy import deepcopy
 from inspect import signature
-from joblib.memory import Memory
-from typeguard import typechecked
-from typing import Union, Optional, Any, Dict
+from platform import machine, platform, python_build, python_version
+from typing import Any, Dict, Optional, Union
 
-# Own modules
-from .branch import Branch
-from .basepredictor import BasePredictor
-from .basetrainer import BaseTrainer
-from .basetransformer import BaseTransformer
-from .nlp import TextCleaner, Tokenizer, Normalizer, Vectorizer
-from .pipeline import Pipeline
-from .data_cleaning import (
-    DropTransformer,
-    FuncTransformer,
-    Cleaner,
-    Gauss,
-    Discretizer,
-    Scaler,
-    Imputer,
-    Encoder,
-    Pruner,
-    Balancer,
+import numpy as np
+import pandas as pd
+from joblib.memory import Memory
+from scipy import stats
+from typeguard import typechecked
+
+from atom.basepredictor import BasePredictor
+from atom.basetrainer import BaseTrainer
+from atom.basetransformer import BaseTransformer
+from atom.branch import Branch
+from atom.data_cleaning import (
+    Balancer, Cleaner, Discretizer, DropTransformer, Encoder,
+    FunctionTransformer, Imputer, Normalizer, Pruner, Scaler,
 )
-from .feature_engineering import (
-    FeatureExtractor,
-    FeatureGenerator,
-    FeatureSelector,
+from atom.feature_engineering import (
+    FeatureExtractor, FeatureGenerator, FeatureSelector,
 )
-from .training import (
-    DirectClassifier,
-    DirectRegressor,
-    SuccessiveHalvingClassifier,
-    SuccessiveHalvingRegressor,
-    TrainSizingClassifier,
-    TrainSizingRegressor,
+from atom.models import MODELS_ENSEMBLES, CustomModel
+from atom.nlp import TextCleaner, TextNormalizer, Tokenizer, Vectorizer
+from atom.pipeline import Pipeline
+from atom.plots import ATOMPlotter
+from atom.training import (
+    DirectClassifier, DirectRegressor, SuccessiveHalvingClassifier,
+    SuccessiveHalvingRegressor, TrainSizingClassifier, TrainSizingRegressor,
 )
-from .models import CustomModel, MODELS_ENSEMBLES
-from .plots import ATOMPlotter
-from .utils import (
-    SCALAR, SEQUENCE_TYPES, X_TYPES, Y_TYPES, flt, lst, divide,
-    infer_task, check_dim, check_scaling, is_multidim, is_sparse,
-    get_pl_name, names_from_estimator, check_is_fitted, variable_return,
-    fit_one, delete, custom_transform, method_to_log, composed, crash,
-    Table, CustomDict,
+from atom.utils import (
+    INT, SCALAR, SEQUENCE_TYPES, X_TYPES, Y_TYPES, CustomDict, Table,
+    __version__, check_dim, check_is_fitted, check_scaling, composed, crash,
+    custom_transform, delete, divide, fit_one, flt, get_pl_name, infer_task,
+    is_multidim, is_sparse, lst, method_to_log, names_from_estimator,
+    variable_return,
 )
 
 
@@ -108,8 +94,19 @@ class ATOM(BasePredictor, ATOMPlotter):
 
         self.task = infer_task(self.y, goal=self.goal)
         self.log(f"Algorithm task: {self.task}.", 1)
+
         if self.n_jobs > 1:
             self.log(f"Parallel processing with {self.n_jobs} cores.", 1)
+        if self.gpu:
+            self.log("GPU training enabled.", 1)
+
+        # System settings only to logger
+        self.log("\nSystem info ====================== >>", 3)
+        self.log(f"Python version: {python_version()}", 3)
+        self.log(f"Python build: {python_build()}", 3)
+        self.log(f"Machine: {machine()}", 3)
+        self.log(f"OS: {platform()}", 3)
+        self.log(f"ATOM version: {__version__}", 3)
 
         self.log("", 1)  # Add empty rows around stats for cleaner look
         self.stats(1)
@@ -146,9 +143,9 @@ class ATOM(BasePredictor, ATOMPlotter):
         else:
             # Branch can be created from current or another one
             if "_from_" in name:
-                new, old = name.split("_from_")
+                new, parent = name.split("_from_")
             else:
-                new, old = name, self._current
+                new, parent = name, self._current
 
             # Check if the new name is valid
             if not new:
@@ -165,13 +162,13 @@ class ATOM(BasePredictor, ATOMPlotter):
                         )
 
             # Check if the parent branch exists
-            if old not in self._branches:
+            if parent not in self._branches:
                 raise ValueError(
                     "The selected branch to split from does not exist! Use "
                     "atom.status() for an overview of the available branches."
                 )
 
-            self._branches[new] = Branch(self, new, parent=self._branches[old])
+            self._branches[new] = Branch(self, new, parent=self._branches[parent])
             self._current = new
             self.log(f"New branch {self._current} successfully created.", 1)
 
@@ -208,7 +205,7 @@ class ATOM(BasePredictor, ATOMPlotter):
     def numerical(self):
         """Names of the numerical features in the dataset."""
         if not is_multidim(self.X):
-            return list(self.X.select_dtypes(include=["number"]).columns)
+            return self.X.select_dtypes(include=["number"]).columns
 
     @property
     def n_numerical(self):
@@ -220,7 +217,7 @@ class ATOM(BasePredictor, ATOMPlotter):
     def categorical(self):
         """Names of the categorical features in the dataset."""
         if not is_multidim(self.X):
-            return list(self.X.select_dtypes(include=["object", "category"]).columns)
+            return self.X.select_dtypes(include=["object", "category"]).columns
 
     @property
     def n_categorical(self):
@@ -337,7 +334,7 @@ class ATOM(BasePredictor, ATOMPlotter):
     def distribution(
         self,
         distributions: Optional[Union[str, SEQUENCE_TYPES]] = None,
-        columns: Optional[Union[int, str, slice, SEQUENCE_TYPES]] = None,
+        columns: Optional[Union[INT, str, slice, SEQUENCE_TYPES]] = None,
     ):
         """Get statistics on column distributions.
 
@@ -413,7 +410,7 @@ class ATOM(BasePredictor, ATOMPlotter):
         self,
         model: Optional[str] = None,
         memory: Optional[Union[bool, str, Memory]] = None,
-        verbose: Optional[int] = None,
+        verbose: Optional[INT] = None,
     ):
         """Export atom's pipeline to a sklearn-like Pipeline object.
 
@@ -576,7 +573,7 @@ class ATOM(BasePredictor, ATOMPlotter):
         obj2cat: bool = True,
         int2uint: bool = False,
         dense2sparse: bool = False,
-        columns: Optional[Union[int, str, slice, SEQUENCE_TYPES]] = None,
+        columns: Optional[Union[INT, str, slice, SEQUENCE_TYPES]] = None,
     ):
         """Converts the columns to the smallest possible matching dtype.
 
@@ -680,7 +677,7 @@ class ATOM(BasePredictor, ATOMPlotter):
         self.log("The column dtypes are successfully converted.", 1)
 
     @composed(crash, method_to_log)
-    def stats(self, _vb: int = -2):
+    def stats(self, _vb: INT = -2):
         """Print basic information about the dataset.
 
         Parameters
@@ -766,7 +763,7 @@ class ATOM(BasePredictor, ATOMPlotter):
         self,
         X: X_TYPES,
         y: Optional[Y_TYPES] = None,
-        verbose: Optional[int] = None,
+        verbose: Optional[INT] = None,
     ):
         """Transform new data through the branch.
 
@@ -882,7 +879,7 @@ class ATOM(BasePredictor, ATOMPlotter):
 
         # Add the estimator to the pipeline
         self.branch.pipeline = pd.concat(
-            [self.pipeline, pd.Series([estimator], name=self._current)],
+            [self.pipeline, pd.Series([estimator], name=self._current, dtype="object")],
             ignore_index=True,
         )
 
@@ -890,7 +887,7 @@ class ATOM(BasePredictor, ATOMPlotter):
     def add(
         self,
         transformer: Any,
-        columns: Optional[Union[int, str, slice, SEQUENCE_TYPES]] = None,
+        columns: Optional[Union[INT, str, slice, SEQUENCE_TYPES]] = None,
         train_only: bool = False,
         **fit_params,
     ):
@@ -934,7 +931,7 @@ class ATOM(BasePredictor, ATOMPlotter):
             self._add_transformer(transformer, columns, train_only, **fit_params)
 
     @composed(crash, method_to_log, typechecked)
-    def apply(self, func: callable, columns: Union[int, str], args=(), **kwargs):
+    def apply(self, func: callable, columns: Union[INT, str], args=(), **kwargs):
         """Apply a function to the dataset.
 
         Transform one column in the dataset using a function (can
@@ -974,10 +971,10 @@ class ATOM(BasePredictor, ATOMPlotter):
             columns = self._get_columns(columns)[0]
 
         kwargs = self._prepare_kwargs(kwargs, ["verbose", "logger"])
-        self._add_transformer(FuncTransformer(func, columns, args, **kwargs))
+        self._add_transformer(FunctionTransformer(func, columns, args, **kwargs))
 
     @composed(crash, method_to_log, typechecked)
-    def drop(self, columns: Union[int, str, slice, SEQUENCE_TYPES], **kwargs):
+    def drop(self, columns: Union[INT, str, slice, SEQUENCE_TYPES], **kwargs):
         """Drop columns from the dataset.
 
         This approach is preferred over dropping columns from the
@@ -1019,8 +1016,8 @@ class ATOM(BasePredictor, ATOMPlotter):
         setattr(self.branch, strategy.lower(), getattr(scaler, strategy.lower()))
 
     @composed(crash, method_to_log)
-    def gauss(self, strategy: str = "yeo-johnson", **kwargs):
-        """Transform the data to follow a Gaussian distribution.
+    def normalize(self, strategy: str = "yeojohnson", **kwargs):
+        """Transform the data to follow a Normal/Gaussian distribution.
 
         This transformation is useful for modeling issues related
         to heteroscedasticity (non-constant variance), or other
@@ -1032,17 +1029,17 @@ class ATOM(BasePredictor, ATOMPlotter):
         See data_cleaning.py for a description of the parameters.
 
         """
-        check_dim(self, "gauss")
+        check_dim(self, "normalize")
         columns = kwargs.pop("columns", None)
-        kwargs = self._prepare_kwargs(kwargs, Gauss().get_params())
-        gauss = Gauss(strategy=strategy, **kwargs)
+        kwargs = self._prepare_kwargs(kwargs, Normalizer().get_params())
+        normalizer = Normalizer(strategy=strategy, **kwargs)
 
-        self._add_transformer(gauss, columns=columns)
+        self._add_transformer(normalizer, columns=columns)
 
         # Attach the estimator attribute to atom's branch
         for attr in ("yeojohnson", "boxcox", "quantile"):
-            if hasattr(gauss, attr):
-                setattr(self.branch, attr, getattr(gauss, attr))
+            if hasattr(normalizer, attr):
+                setattr(self.branch, attr, getattr(normalizer, attr))
 
     @composed(crash, method_to_log, typechecked)
     def clean(
@@ -1066,7 +1063,7 @@ class ATOM(BasePredictor, ATOMPlotter):
             - Drop columns with minimum cardinality.
             - Drop duplicate rows.
             - Drop rows with missing values in the target column.
-            - Encode the target column (only for classification tasks).
+            - Encode the target column (can't be True for regression tasks).
 
         See data_cleaning.py for a description of the parameters.
 
@@ -1090,7 +1087,7 @@ class ATOM(BasePredictor, ATOMPlotter):
         self._add_transformer(cleaner, columns=columns)
 
         if cleaner.mapping:
-            self.branch.mapping.insert(-1, self.target, cleaner.mapping)
+            self.mapping.insert(-1, self.target, cleaner.mapping)
 
     @composed(crash, method_to_log, typechecked)
     def impute(
@@ -1130,7 +1127,7 @@ class ATOM(BasePredictor, ATOMPlotter):
     def discretize(
         self,
         strategy: str = "quantile",
-        bins: Union[int, SEQUENCE_TYPES, dict] = 5,
+        bins: Union[INT, SEQUENCE_TYPES, dict] = 5,
         labels: Optional[Union[SEQUENCE_TYPES, dict]] = None,
         **kwargs,
     ):
@@ -1154,8 +1151,8 @@ class ATOM(BasePredictor, ATOMPlotter):
     def encode(
         self,
         strategy: str = "LeaveOneOut",
-        max_onehot: Optional[int] = 10,
-        ordinal: Optional[Dict[Union[int, str], SEQUENCE_TYPES]] = None,
+        max_onehot: Optional[INT] = 10,
+        ordinal: Optional[Dict[Union[INT, str], SEQUENCE_TYPES]] = None,
         frac_to_other: Optional[SCALAR] = None,
         **kwargs,
     ):
@@ -1190,13 +1187,13 @@ class ATOM(BasePredictor, ATOMPlotter):
         self._add_transformer(encoder, columns=columns)
 
         # Add mapping of the encoded columns to the branch
-        for key, value in encoder.mapping.items():
-            self.branch.mapping.insert(-1, key, value)
+        self.mapping.update(encoder.mapping)
+        self.mapping = self.mapping[[c for c in self.columns if c in self.mapping]]
 
     @composed(crash, method_to_log, typechecked)
     def prune(
         self,
-        strategy: Union[str, SEQUENCE_TYPES] = "z-score",
+        strategy: Union[str, SEQUENCE_TYPES] = "zscore",
         method: Union[SCALAR, str] = "drop",
         max_sigma: SCALAR = 3,
         include_target: bool = False,
@@ -1231,11 +1228,11 @@ class ATOM(BasePredictor, ATOMPlotter):
 
         # Attach the estimator attribute to atom's branch
         for strat in lst(strategy):
-            if strat.lower() != "z-score":
+            if strat.lower() != "zscore":
                 setattr(self.branch, strat.lower(), getattr(pruner, strat.lower()))
 
     @composed(crash, method_to_log, typechecked)
-    def balance(self, strategy: str = "ADASYN", **kwargs):
+    def balance(self, strategy: str = "adasyn", **kwargs):
         """Balance the number of rows per class in the target column.
 
         Use only for classification tasks. The estimator created by
@@ -1357,7 +1354,7 @@ class ATOM(BasePredictor, ATOMPlotter):
         self.branch.quadgrams = tokenizer.quadgrams
 
     @composed(crash, method_to_log, typechecked)
-    def normalize(
+    def textnormalize(
         self,
         stopwords: Union[bool, str] = True,
         custom_stopwords: Optional[SEQUENCE_TYPES] = None,
@@ -1370,15 +1367,16 @@ class ATOM(BasePredictor, ATOMPlotter):
         Convert words to a more uniform standard. The transformations
         are applied on the column named `corpus`, in the same order the
         parameters are presented. If there is no column with that name,
-        an exception is raised.
+        an exception is raised. If the provided documents are strings,
+        words are separated by spaces.
 
         See nlp.py for a description of the parameters.
 
         """
-        check_dim(self, "normalize")
+        check_dim(self, "textnormalize")
         columns = kwargs.pop("columns", None)
-        kwargs = self._prepare_kwargs(kwargs, Normalizer().get_params())
-        normalizer = Normalizer(
+        kwargs = self._prepare_kwargs(kwargs, TextNormalizer().get_params())
+        normalizer = TextNormalizer(
             stopwords=stopwords,
             custom_stopwords=custom_stopwords,
             stem=stem,
@@ -1389,12 +1387,15 @@ class ATOM(BasePredictor, ATOMPlotter):
         self._add_transformer(normalizer, columns=columns)
 
     @composed(crash, method_to_log, typechecked)
-    def vectorize(self, strategy: str = "BOW", **kwargs):
+    def vectorize(self, strategy: str = "bow", return_sparse: bool = True, **kwargs):
         """Vectorize the corpus.
 
         Transform the corpus into meaningful vectors of numbers. The
         transformation is applied on the column named `corpus`. If
-        there is no column with that name, an exception is raised.
+        there is no column with that name, an exception is raised. The
+        transformed columns are named after the word they are embedding
+        (if the column is already present in the provided dataset,
+        `_[strategy]` is added behind the name).
 
         See nlp.py for a description of the parameters.
 
@@ -1402,7 +1403,11 @@ class ATOM(BasePredictor, ATOMPlotter):
         check_dim(self, "normalize")
         columns = kwargs.pop("columns", None)
         kwargs = self._prepare_kwargs(kwargs, Vectorizer().get_params())
-        vectorizer = Vectorizer(strategy=strategy, **kwargs)
+        vectorizer = Vectorizer(
+            strategy=strategy,
+            return_sparse=return_sparse,
+            **kwargs,
+        )
 
         self._add_transformer(vectorizer, columns=columns)
 
@@ -1449,17 +1454,16 @@ class ATOM(BasePredictor, ATOMPlotter):
     @composed(crash, method_to_log, typechecked)
     def feature_generation(
         self,
-        strategy: str = "DFS",
-        n_features: Optional[int] = None,
+        strategy: str = "dfs",
+        n_features: Optional[INT] = None,
         operators: Optional[Union[str, SEQUENCE_TYPES]] = None,
         **kwargs,
     ):
         """Apply automated feature engineering.
 
-        Use Deep feature Synthesis or a genetic algorithm to create
-        new combinations of existing features to capture the non-linear
-        relations between the original features. Attributes created by
-        the class are attached to atom.
+        Create new combinations of existing features to capture the
+        non-linear relations between the original features. Attributes
+        created by the class are attached to atom.
 
         See feature_engineering.py for a description of the parameters.
 
@@ -1477,8 +1481,8 @@ class ATOM(BasePredictor, ATOMPlotter):
         self._add_transformer(feature_generator, columns=columns)
 
         # Attach the genetic attributes to atom's branch
-        if strategy.lower() in ("gfg", "genetic"):
-            self.branch.symbolic_transformer = feature_generator.symbolic_transformer
+        if strategy.lower() == "gfg":
+            self.branch.gfg = feature_generator.gfg
             self.branch.genetic_features = feature_generator.genetic_features
 
     @composed(crash, method_to_log, typechecked)
@@ -1494,13 +1498,9 @@ class ATOM(BasePredictor, ATOMPlotter):
         """Apply feature selection techniques.
 
         Remove features according to the selected strategy. Ties
-        between features with equal scores are broken in an
-        unspecified way. Additionally, removes features with too low
-        variance and finds pairs of collinear features based on the
-        Pearson correlation coefficient. For each pair above the
-        specified limit (in terms of absolute value), it removes one
-        of the two. Plotting methods and attributes created by the
-        class are attached to atom.
+        between features with equal scores are broken in an unspecified
+        way. Additionally, remove multicollinear and low variance
+        features.
 
         See feature_engineering.py for a description of the parameters.
 
@@ -1509,7 +1509,7 @@ class ATOM(BasePredictor, ATOMPlotter):
         if isinstance(strategy, str):
             if strategy.lower() == "univariate" and solver is None:
                 solver = "f_classif" if self.goal == "class" else "f_regression"
-            elif strategy.lower() in ("sfm", "rfe", "rfecv", "sfs"):
+            elif strategy.lower() not in ("univariate", "pca"):
                 if solver is None and self.winner:
                     solver = self.winner.estimator
                 elif isinstance(solver, str):
@@ -1518,7 +1518,7 @@ class ATOM(BasePredictor, ATOMPlotter):
                         solver += f"_{self.goal}"
 
             # If the run method was called before, use the main metric
-            if strategy.lower() in ("rfecv", "sfs"):
+            if strategy.lower() not in ("univariate", "pca", "sfm", "rfe"):
                 if self._metric and "scoring" not in kwargs:
                     kwargs["scoring"] = self._metric[0]
 
@@ -1612,14 +1612,22 @@ class ATOM(BasePredictor, ATOMPlotter):
             # Catch errors and pass them to atom's attribute
             for model, error in trainer.errors.items():
                 self._errors[model] = error
-                self._models.pop(model, None)
+                self._models.pop(model)
+
+        # Overwrite models with same name as new ones
+        for model in trainer._models:
+            if model in self._models:
+                self._models.pop(model)
+                self.log(
+                    f"\nWarning: Consecutive runs of the same model ({model}) "
+                    "were detected. The former model has been overwritten.", 3
+                )
 
-        # Update attributes
         self._models.update(trainer._models)
         self._metric = trainer._metric
 
         for model in self._models.values():
-            self._errors.pop(model.name, None)  # Remove model from errors (if there)
+            self._errors.pop(model.name)  # Remove model from errors (if there)
             model.T = self  # Change the model's parent class from trainer to atom
 
     @composed(crash, method_to_log, typechecked)
@@ -1630,11 +1638,11 @@ class ATOM(BasePredictor, ATOMPlotter):
         greater_is_better: Union[bool, SEQUENCE_TYPES] = True,
         needs_proba: Union[bool, SEQUENCE_TYPES] = False,
         needs_threshold: Union[bool, SEQUENCE_TYPES] = False,
-        n_calls: Union[int, SEQUENCE_TYPES] = 0,
-        n_initial_points: Union[int, SEQUENCE_TYPES] = 5,
+        n_calls: Union[INT, SEQUENCE_TYPES] = 0,
+        n_initial_points: Union[INT, SEQUENCE_TYPES] = 5,
         est_params: Optional[dict] = None,
         bo_params: Optional[dict] = None,
-        n_bootstrap: Union[int, SEQUENCE_TYPES] = 0,
+        n_bootstrap: Union[INT, SEQUENCE_TYPES] = 0,
         **kwargs,
     ):
         """Fit the models in a direct fashion.
@@ -1669,12 +1677,12 @@ class ATOM(BasePredictor, ATOMPlotter):
         greater_is_better: Union[bool, SEQUENCE_TYPES] = True,
         needs_proba: Union[bool, SEQUENCE_TYPES] = False,
         needs_threshold: Union[bool, SEQUENCE_TYPES] = False,
-        skip_runs: int = 0,
-        n_calls: Union[int, SEQUENCE_TYPES] = 0,
-        n_initial_points: Union[int, SEQUENCE_TYPES] = 5,
+        skip_runs: INT = 0,
+        n_calls: Union[INT, SEQUENCE_TYPES] = 0,
+        n_initial_points: Union[INT, SEQUENCE_TYPES] = 5,
         est_params: Optional[dict] = None,
         bo_params: Optional[dict] = None,
-        n_bootstrap: Union[int, SEQUENCE_TYPES] = 0,
+        n_bootstrap: Union[INT, SEQUENCE_TYPES] = 0,
         **kwargs,
     ):
         """Fit the models in a successive halving fashion.
@@ -1715,12 +1723,12 @@ class ATOM(BasePredictor, ATOMPlotter):
         greater_is_better: Union[bool, SEQUENCE_TYPES] = True,
         needs_proba: Union[bool, SEQUENCE_TYPES] = False,
         needs_threshold: Union[bool, SEQUENCE_TYPES] = False,
-        train_sizes: Union[int, SEQUENCE_TYPES] = 5,
-        n_calls: Union[int, SEQUENCE_TYPES] = 0,
-        n_initial_points: Union[int, SEQUENCE_TYPES] = 5,
+        train_sizes: Union[INT, SEQUENCE_TYPES] = 5,
+        n_calls: Union[INT, SEQUENCE_TYPES] = 0,
+        n_initial_points: Union[INT, SEQUENCE_TYPES] = 5,
         est_params: Optional[dict] = None,
         bo_params: Optional[dict] = None,
-        n_bootstrap: Union[int, SEQUENCE_TYPES] = 0,
+        n_bootstrap: Union[INT, SEQUENCE_TYPES] = 0,
         **kwargs,
     ):
         """Fit the models in a train sizing fashion.
