@@ -7,20 +7,18 @@ Description: Module containing the BasePredictor class.
 
 """
 
-# Standard packages
+from typing import Any, Optional, Union
+
 import mlflow
-import numpy as np
 import pandas as pd
 from typeguard import typechecked
-from typing import Union, Optional, Any
 
-# Own modules
-from .branch import Branch
-from .models import MODELS, Stacking, Voting
-from .utils import (
-    SEQUENCE_TYPES, X_TYPES, Y_TYPES, DF_ATTRS, flt, lst,
-    check_is_fitted, divide, get_best_score, delete,
-    method_to_log, composed, crash, CustomDict,
+from atom.branch import Branch
+from atom.models import MODELS, Stacking, Voting
+from atom.utils import (
+    DF_ATTRS, FLOAT, SEQUENCE_TYPES, X_TYPES, Y_TYPES, CustomDict,
+    check_is_fitted, composed, crash, delete, divide, flt, get_best_score, lst,
+    method_to_log,
 )
 
 
@@ -184,12 +182,18 @@ class BasePredictor:
         """Return the errors encountered during model training."""
         return self._errors
 
+    @property
+    def winners(self):
+        """Return the model names ordered by performance."""
+        if self._models:  # Returns None if not fitted
+            models = sorted(self._models.values(), key=lambda x: get_best_score(x))
+            return [m.name for m in models[::-1]]
+
     @property
     def winner(self):
         """Return the best performing model."""
         if self._models:  # Returns None if not fitted
-            best_index = np.argmax([get_best_score(m) for m in self._models.values()])
-            return self._models[best_index]  # CustomDict can select item from index
+            return self._models[self.winners[0]]
 
     @property
     def results(self):
@@ -470,7 +474,7 @@ class BasePredictor:
             )
 
     def _get_models(self, models=None, ensembles=True):
-        """Return models in the pipeline."""
+        """Return the names of models in the pipeline."""
         if not models:
             if self._models:
                 to_return = lst(self.models).copy()
@@ -507,13 +511,14 @@ class BasePredictor:
                 - module: The estimator's module.
                 - needs_scaling: Whether the model requires feature scaling.
                 - accepts_sparse: Whether the model supports sparse matrices.
+                - supports_gpu: Whether the model has GPU support.
 
         """
-        overview = pd.DataFrame()
+        rows = []
         for model in MODELS.values():
             m = model(self, fast_init=True)
             if self.goal in m.goal:
-                overview = overview.append(
+                rows.append(
                     {
                         "acronym": m.acronym,
                         "fullname": m.fullname,
@@ -521,11 +526,11 @@ class BasePredictor:
                         "module": m.est_class.__module__,
                         "needs_scaling": str(m.needs_scaling),
                         "accepts_sparse": str(m.accepts_sparse),
-                    },
-                    ignore_index=True,
+                        "supports_gpu": str(m.supports_gpu),
+                    }
                 )
 
-        return overview
+        return pd.DataFrame(rows)
 
     @composed(crash, method_to_log)
     def clear(self):
@@ -574,7 +579,7 @@ class BasePredictor:
         self,
         metric: Optional[Union[str, callable, SEQUENCE_TYPES]] = None,
         dataset: str = "test",
-        threshold: float = 0.5,
+        threshold: FLOAT = 0.5,
         sample_weight: Optional[SEQUENCE_TYPES] = None,
     ):
         """Get all models' scores for the provided metrics.
@@ -756,6 +761,13 @@ class BasePredictor:
         if not name.lower().startswith("stack"):
             name = f"Stack{name}"
 
+        if name in self._models:
+            raise ValueError(
+                "Invalid value for the name parameter. It seems a model with "
+                f"the name {name} already exists. Add a different name to "
+                "train multiple Stacking models within the same instance."
+            )
+
         if isinstance(kwargs.get("final_estimator"), str):
             if kwargs["final_estimator"] not in MODELS:
                 raise ValueError(
@@ -779,6 +791,9 @@ class BasePredictor:
 
         self[name].fit()
 
+        if self.experiment:
+            mlflow.end_run()
+
     @composed(crash, method_to_log, typechecked)
     def voting(
         self,
@@ -814,9 +829,19 @@ class BasePredictor:
         if not name.lower().startswith("vote"):
             name = f"Vote{name}"
 
+        if name in self._models:
+            raise ValueError(
+                "Invalid value for the name parameter. It seems a model with "
+                f"the name {name} already exists. Add a different name to "
+                "train multiple Voting models within the same instance."
+            )
+
         self._models[name] = Voting(self, name, models=self._models[models], **kwargs)
 
         if self.experiment:
             self[name]._run = mlflow.start_run(run_name=self[name].name)
 
         self[name].fit()
+
+        if self.experiment:
+            mlflow.end_run()
