@@ -7,32 +7,31 @@ Description: Module containing the BaseTransformer class.
 
 """
 
-# Standard packages
+import importlib
+import multiprocessing
 import os
-import dill
 import random
-import mlflow
 import warnings
+from copy import deepcopy
+from typing import Optional, Union
+
+import dill
+import mlflow
 import numpy as np
 import pandas as pd
-import multiprocessing
-from copy import deepcopy
-from typeguard import typechecked
-from typing import Union, Optional
 from sklearn.model_selection import train_test_split
+from typeguard import typechecked
 
-# Own modules
-from .utils import (
-    SEQUENCE, X_TYPES, Y_TYPES, lst, to_df, to_series, merge,
-    prepare_logger, composed, crash, method_to_log,
+from atom.utils import (
+    INT, SCALAR, SEQUENCE, X_TYPES, Y_TYPES, composed, crash, lst, merge,
+    method_to_log, prepare_logger, to_df, to_series,
 )
 
 
 class BaseTransformer:
     """Base class for estimators in the package.
 
-    Contains shared properties (n_jobs, verbose, warnings, logger,
-    random_state) and standard methods across estimators.
+    Contains shared properties and standard methods across transformers.
 
     Parameters
     ----------
@@ -43,11 +42,20 @@ class BaseTransformer:
             - warnings: Whether to show or suppress encountered warnings.
             - logger: Name of the log file or Logger object.
             - experiment: Name of the mlflow experiment used for tracking.
+            - gpu: Whether to train the pipeline on GPU.
             - random_state: Seed used by the random number generator.
 
     """
 
-    attrs = ["n_jobs", "verbose", "warnings", "logger", "experiment", "random_state"]
+    attrs = [
+        "n_jobs",
+        "verbose",
+        "warnings",
+        "logger",
+        "experiment",
+        "gpu",
+        "random_state",
+    ]
 
     def __init__(self, **kwargs):
         """Update the properties with the provided kwargs."""
@@ -129,15 +137,30 @@ class BaseTransformer:
     def experiment(self, value):
         self._experiment = value
         if value:
+            mlflow.sklearn.autolog(disable=True)
             mlflow.set_experiment(value)
 
+    @property
+    def gpu(self):
+        return self._gpu
+
+    @gpu.setter
+    @typechecked
+    def gpu(self, value: Union[bool, str]):
+        if isinstance(value, str) and value.lower() != "force":
+            raise ValueError(
+                "Invalid value for the gpu parameter. Only True, "
+                f"False and 'force' are valid values, got {value}."
+            )
+        self._gpu = value
+
     @property
     def random_state(self):
         return self._random_state
 
     @random_state.setter
     @typechecked
-    def random_state(self, value: Optional[int]):
+    def random_state(self, value: Optional[INT]):
         if value and value < 0:
             raise ValueError(
                 "Invalid value for the random_state parameter. "
@@ -149,9 +172,43 @@ class BaseTransformer:
 
     # Methods ====================================================== >>
 
+    def _get_gpu(self, estimator, module="cuml"):
+        """Get a GPU or CPU estimator depending on availability.
+
+        Parameters
+        ----------
+        estimator: estimator
+            Class to get GPU implementation from.
+
+        module: str, optional (default="cuml")
+            Module from which to get the GPU estimator.
+
+        Returns
+        -------
+        estimator
+            Provided estimator or cuml implementation.
+
+        """
+        if self.gpu:
+            try:
+                return getattr(importlib.import_module(module), estimator.__name__)
+            except ModuleNotFoundError:
+                if str(self.gpu).lower() == "force":
+                    raise ModuleNotFoundError(
+                        "It looks like cuml is not installed. Refer to the package's "
+                        "documentation to learn how to utilize the machine's GPU."
+                    )
+                else:
+                    self.log(
+                        f" --> Unable to import {module}.{estimator.__name__}. "
+                        "Using CPU implementation instead.", 1
+                    )
+
+        return estimator
+
     @staticmethod
     @typechecked
-    def _prepare_input(X: X_TYPES, y: Optional[Y_TYPES] = None):
+    def _prepare_input(X: Optional[X_TYPES] = None, y: Optional[Y_TYPES] = None):
         """Prepare the input data.
 
         Convert X and y to pandas (if not already) and perform standard
@@ -159,7 +216,7 @@ class BaseTransformer:
 
         Parameters
         ----------
-        X: dataframe-like
+        X: dataframe-like or None, optional (default=None)
             Feature set with shape=(n_samples, n_features).
 
         y: int, str, sequence or None, optional (default=None)
@@ -177,23 +234,24 @@ class BaseTransformer:
             Target column corresponding to X.
 
         """
-        # If data has more than 2 dimensions and is not a text corpus,
-        # create a dataframe with one multidimensional column
-        array = np.array(X)
-        if array.ndim > 2 and not isinstance(array[0, 0, 0], str):
-            X = pd.DataFrame({"multidim feature": [row for row in X]})
-        else:
-            X = to_df(deepcopy(X))  # Make copy to not overwrite mutable arguments
+        if X is not None:
+            # If data has more than 2 dimensions and is not a text corpus,
+            # create a dataframe with one multidimensional column
+            array = np.array(X)
+            if array.ndim > 2 and not isinstance(array[0, 0, 0], str):
+                X = pd.DataFrame({"multidim feature": [row for row in X]})
+            else:
+                X = to_df(deepcopy(X))  # Make copy to not overwrite mutable arguments
 
-            # If text dataset, change the name of the column to corpus
-            if list(X.columns) == ["feature 1"] and X[X.columns[0]].dtype == "object":
-                X = X.rename(columns={X.columns[0]: "corpus"})
+                # If text dataset, change the name of the column to corpus
+                if list(X.columns) == ["x0"] and X[X.columns[0]].dtype == "object":
+                    X = X.rename(columns={X.columns[0]: "corpus"})
 
-            # Convert all column names to str
-            X.columns = [str(col) for col in X.columns]
+                # Convert all column names to str
+                X.columns = [str(col) for col in X.columns]
 
         # Prepare target column
-        if isinstance(y, (dict, *SEQUENCE)):
+        if isinstance(y, SEQUENCE):
             if not isinstance(y, pd.Series):
                 # Check that y is one-dimensional
                 ndim = np.array(y).ndim
@@ -201,25 +259,32 @@ class BaseTransformer:
                     raise ValueError(f"y should be one-dimensional, got ndim={ndim}.")
 
                 # Check X and y have the same number of rows
-                if len(X) != len(y):
+                if X is not None and len(X) != len(y):
                     raise ValueError(
                         "X and y don't have the same number of rows,"
                         f" got len(X)={len(X)} and len(y)={len(y)}."
                     )
 
-                y = to_series(y, index=X.index)
+                y = to_series(y, index=getattr(X, "index", None))
 
-            elif not X.index.equals(y.index):  # Compare indices
+            elif X is not None and not X.index.equals(y.index):
                 raise ValueError("X and y don't have the same indices!")
 
         elif isinstance(y, str):
-            if y not in X.columns:
-                raise ValueError(f"Column {y} not found in X!")
+            if X is not None:
+                if y not in X.columns:
+                    raise ValueError(f"Column {y} not found in X!")
 
-            return X.drop(y, axis=1), X[y]
+                X, y = X.drop(y, axis=1), X[y]
+
+            else:
+                raise ValueError("X can't be None when y is a string.")
 
         elif isinstance(y, int):
-            return X.drop(X.columns[y], axis=1), X[X.columns[y]]
+            if X is None:
+                raise ValueError("X can't be None when y is a int.")
+
+            X, y = X.drop(X.columns[y], axis=1), X[X.columns[y]]
 
         return X, y
 
@@ -522,7 +587,7 @@ class BaseTransformer:
         return data, idx, holdout
 
     @composed(crash, typechecked)
-    def log(self, msg: Union[int, float, str], level: int = 0):
+    def log(self, msg: Union[SCALAR, str], level: INT = 0):
         """Print and save output to log file.
 
         Parameters
@@ -568,6 +633,7 @@ class BaseTransformer:
             filename = filename.replace("auto", self.__class__.__name__)
 
         with open(filename, "wb") as f:
+            dill.settings["recurse"] = True
             dill.dump(self, f)  # Dill replaces pickle to dump lambdas
 
         # Restore the data to the attributes
