@@ -7,19 +7,18 @@ Description: Module containing the Branch class.
 
 """
 
-# Standard packages
-import pandas as pd
-from inspect import signature
 from copy import copy
-from typeguard import typechecked
+from inspect import signature
 from typing import Optional
 
-# Own modules
-from .basetransformer import BaseTransformer
-from .models import MODELS_ENSEMBLES
-from .utils import (
-    X_TYPES, SEQUENCE_TYPES, flt, merge, to_df, to_series, is_multidim,
-    custom_transform, composed, crash, method_to_log, CustomDict,
+import pandas as pd
+from typeguard import typechecked
+
+from atom.basetransformer import BaseTransformer
+from atom.models import MODELS_ENSEMBLES
+from atom.utils import (
+    SEQUENCE_TYPES, X_TYPES, CustomDict, composed, crash, custom_transform,
+    flt, is_multidim, merge, method_to_log, to_df, to_series,
 )
 
 
@@ -45,6 +44,9 @@ class Branch:
     name: str
         Name of the branch.
 
+    parent: str
+        Name of the parent branch.
+
     mapping: CustomDict, optional (default={})
         Encoded values and their respective mapping. The column name is
         the key to its mapping dictionary. Only for columns mapped to
@@ -61,6 +63,7 @@ class Branch:
     def __init__(self, *args, parent=None):
         self.T = args[0]
         self.name = args[1]
+        self.parent = self.T._current
         self.mapping = CustomDict()
         self.pipeline = pd.Series(data=[], name=self.name, dtype="object")
         self.feature_importance = None
@@ -73,6 +76,8 @@ class Branch:
         # _holdout is always reset since it wouldn't recalculate if
         # changes were made to the pipeline
         if parent:
+            self.parent = parent.name
+
             # Copy the branch attrs and point to the rest
             for attr in ("_data", "_idx", "mapping", "pipeline", "feature_importance"):
                 setattr(self, attr, copy(getattr(parent, attr)))
@@ -97,84 +102,6 @@ class Branch:
 
         return out
 
-    # Utility methods ============================================== >>
-
-    def _get_attrs(self):
-        """Get properties and attributes to call from parent."""
-        attrs = []
-        for p in dir(self):
-            if (
-                p in vars(self) and p not in ("T", "name", "_data", "_idx", "_holdout")
-                or isinstance(getattr(Branch, p, None), property)
-            ):
-                attrs.append(p)
-
-        return attrs
-
-    def _get_depending_models(self):
-        """Return the models that are dependent on this branch."""
-        return [m.name for m in self.T._models.values() if m.branch is self]
-
-    @composed(crash, method_to_log, typechecked)
-    def delete(self, name: Optional[str] = None):
-        """Delete the branch and all the models in it."""
-        if name is None:
-            name = self.name
-
-        if name == "og":
-            raise PermissionError(
-                "The og branch is an internal branch and can not be deleted!"
-            )
-        elif name not in self.T._branches:
-            raise ValueError(f"Branch {name} not found!")
-        elif len(self.T._branches.min("og")) == 1:
-            raise PermissionError("Can't delete the last branch!")
-        else:
-            branch = self.T._branches[name]
-
-            # Delete all depending models
-            depending_models = branch._get_depending_models()
-            if depending_models:
-                self.T.delete(depending_models)
-
-            # If this is the last og branch, create a new one
-            if self.T._get_og_branches() == [branch]:
-                self.T._branches.insert(0, "og", Branch(self.T, "og", parent=self))
-
-            # Reset the current branch
-            if branch.name == self.T._current:
-                self.T._current = list(self.T._branches.min("og"))[0]
-
-            self.T._branches.pop(branch.name)
-            self.T.log(f"Branch {branch.name} successfully deleted.", 1)
-
-    @composed(crash, method_to_log, typechecked)
-    def rename(self, name: str):
-        """Change the name of the branch."""
-        if not name:
-            raise ValueError("A branch can't have an empty name!")
-        elif name in self.T._branches:
-            raise ValueError(f"Branch {self.T._branches[name].name} already exists!")
-        else:
-            for model in MODELS_ENSEMBLES.values():
-                if name.lower().startswith(model.acronym.lower()):
-                    raise ValueError(
-                        "Invalid name for the branch. The name of a branch may "
-                        f"not begin with a model's acronym, and {model.acronym} "
-                        f"is the acronym of the {model.fullname} model."
-                    )
-
-        self.name = name
-        self.pipeline.name = name
-        self.T._branches[name] = self.T._branches.pop(self.T._current)
-        self.T.log(f"Branch {self.T._current} is renamed to {name}.", 1)
-        self.T._current = name
-
-    @composed(crash, method_to_log)
-    def status(self):
-        """Get an overview of the pipeline and models in the branch."""
-        self.T.log(str(self))
-
     # Data properties ============================================== >>
 
     def _check_setter(self, name, value):
@@ -397,7 +324,7 @@ class Branch:
     @property
     def columns(self):
         """Name of all the columns."""
-        return list(self._data.columns)
+        return self._data.columns
 
     @property
     def n_columns(self):
@@ -418,3 +345,81 @@ class Branch:
     def target(self):
         """Name of the target column."""
         return self.columns[-1]
+
+    # Utility methods ============================================== >>
+
+    def _get_attrs(self):
+        """Get properties and attributes to call from parent."""
+        attrs = []
+        for p in dir(self):
+            if (
+                p in vars(self) and p not in ("T", "name", "_data", "_idx", "_holdout")
+                or isinstance(getattr(Branch, p, None), property)
+            ):
+                attrs.append(p)
+
+        return attrs
+
+    def _get_depending_models(self):
+        """Return the models that are dependent on this branch."""
+        return [m.name for m in self.T._models.values() if m.branch is self]
+
+    @composed(crash, method_to_log, typechecked)
+    def delete(self, name: Optional[str] = None):
+        """Delete the branch and all the models in it."""
+        if name is None:
+            name = self.name
+
+        if name == "og":
+            raise PermissionError(
+                "The og branch is an internal branch and can not be deleted!"
+            )
+        elif name not in self.T._branches:
+            raise ValueError(f"Branch {name} not found!")
+        elif len(self.T._branches.min("og")) == 1:
+            raise PermissionError("Can't delete the last branch!")
+        else:
+            branch = self.T._branches[name]
+
+            # Delete all depending models
+            depending_models = branch._get_depending_models()
+            if depending_models:
+                self.T.delete(depending_models)
+
+            # If this is the last og branch, create a new one
+            if self.T._get_og_branches() == [branch]:
+                self.T._branches.insert(0, "og", Branch(self.T, "og", parent=self))
+
+            # Reset the current branch
+            if branch.name == self.T._current:
+                self.T._current = list(self.T._branches.min("og"))[0]
+
+            self.T._branches.pop(branch.name)
+            self.T.log(f"Branch {branch.name} successfully deleted.", 1)
+
+    @composed(crash, method_to_log, typechecked)
+    def rename(self, name: str):
+        """Change the name of the branch."""
+        if not name:
+            raise ValueError("A branch can't have an empty name!")
+        elif name in self.T._branches:
+            raise ValueError(f"Branch {self.T._branches[name].name} already exists!")
+        else:
+            for model in MODELS_ENSEMBLES.values():
+                if name.lower().startswith(model.acronym.lower()):
+                    raise ValueError(
+                        "Invalid name for the branch. The name of a branch may "
+                        f"not begin with a model's acronym, and {model.acronym} "
+                        f"is the acronym of the {model.fullname} model."
+                    )
+
+        self.name = name
+        self.pipeline.name = name
+        self.T._branches[name] = self.T._branches.pop(self.T._current)
+        self.T.log(f"Branch {self.T._current} is renamed to {name}.", 1)
+        self.T._current = name
+
+    @composed(crash, method_to_log)
+    def status(self):
+        """Get an overview of the pipeline and models in the branch."""
+        self.T.log(str(self))
