@@ -7,108 +7,48 @@ Description: Module containing utility constants, functions and classes.
 
 """
 
-# Standard packages
-import sys
+import logging
 import math
 import pprint
-import logging
+import sys
+from collections import deque
+from collections.abc import MutableMapping
+from copy import copy
+from datetime import datetime
+from functools import wraps
+from inspect import signature
+from typing import Union
+
+import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
-from copy import copy
-from typing import Union
+from matplotlib.gridspec import GridSpec
 from scipy import sparse
 from shap import Explainer
-from functools import wraps
-from collections import deque
-from datetime import datetime
-from inspect import signature
-from collections.abc import MutableMapping
-from sklearn.preprocessing import (
-    StandardScaler,
-    MinMaxScaler,
-    MaxAbsScaler,
-    RobustScaler,
-)
-
-from sklearn.ensemble import IsolationForest
-from sklearn.covariance import EllipticEnvelope
-from sklearn.neighbors import LocalOutlierFactor
-from sklearn.svm import OneClassSVM
-from sklearn.cluster import DBSCAN, OPTICS
-from sklearn.utils import _print_elapsed_time
-
-# Encoders
-from category_encoders.backward_difference import BackwardDifferenceEncoder
-from category_encoders.basen import BaseNEncoder
-from category_encoders.binary import BinaryEncoder
-from category_encoders.cat_boost import CatBoostEncoder
-from category_encoders.helmert import HelmertEncoder
-from category_encoders.james_stein import JamesSteinEncoder
-from category_encoders.leave_one_out import LeaveOneOutEncoder
-from category_encoders.m_estimate import MEstimateEncoder
-from category_encoders.ordinal import OrdinalEncoder
-from category_encoders.polynomial import PolynomialEncoder
-from category_encoders.sum_coding import SumEncoder
-from category_encoders.target_encoder import TargetEncoder
-from category_encoders.woe import WOEEncoder
-
-# Balancers
-from imblearn.under_sampling import (
-    CondensedNearestNeighbour,
-    EditedNearestNeighbours,
-    RepeatedEditedNearestNeighbours,
-    AllKNN,
-    InstanceHardnessThreshold,
-    NearMiss,
-    NeighbourhoodCleaningRule,
-    OneSidedSelection,
-    RandomUnderSampler,
-    TomekLinks,
-)
-from imblearn.over_sampling import (
-    ADASYN,
-    BorderlineSMOTE,
-    KMeansSMOTE,
-    RandomOverSampler,
-    SMOTE,
-    SMOTENC,
-    SMOTEN,
-    SVMSMOTE,
-)
-from imblearn.combine import SMOTEENN, SMOTETomek
-
-# Sklearn
-from sklearn.metrics import (
-    SCORERS,
-    make_scorer,
-    confusion_matrix,
-    matthews_corrcoef,
-)
-from sklearn.utils import _safe_indexing
 from sklearn.inspection._partial_dependence import (
-    _grid_from_X,
-    _partial_dependence_brute,
+    _grid_from_X, _partial_dependence_brute,
 )
-
-# Plotting
-import matplotlib.pyplot as plt
-from matplotlib.gridspec import GridSpec
+from sklearn.metrics import (
+    SCORERS, confusion_matrix, make_scorer, matthews_corrcoef,
+)
+from sklearn.utils import _print_elapsed_time, _safe_indexing
 
 
 # Global constants ================================================= >>
 
+__version__ = "4.14.0"
+
 SEQUENCE = (list, tuple, np.ndarray, pd.Series)
 
 # Variable types
-SCALAR = Union[int, float]
+INT = Union[int, np.integer]
+FLOAT = Union[float, np.float]
+SCALAR = Union[INT, FLOAT]
 SEQUENCE_TYPES = Union[SEQUENCE]
 X_TYPES = Union[iter, dict, list, tuple, np.ndarray, sparse.spmatrix, pd.DataFrame]
-Y_TYPES = Union[int, str, SEQUENCE_TYPES]
+Y_TYPES = Union[INT, str, SEQUENCE_TYPES]
 
-# Non-sklearn models
-OPTIONAL_PACKAGES = dict(XGB="xgboost", LGB="lightgbm", CatB="catboost")
-
-# Attributes shared betwen atom and a pd.DataFrame
+# Attributes shared between atom and a pd.DataFrame
 DF_ATTRS = (
     "size",
     "head",
@@ -120,6 +60,9 @@ DF_ATTRS = (
     "dtypes",
     "at",
     "iat",
+    "memory_usage",
+    "empty",
+    "ndim",
 )
 
 # List of custom metrics for the evaluate method
@@ -155,68 +98,6 @@ SCORERS_ACRONYMS = dict(
     gamma="neg_mean_gamma_deviance",
 )
 
-# All available scaling strategies
-SCALING_STRATS = dict(
-    standard=StandardScaler,
-    minmax=MinMaxScaler,
-    maxabs=MaxAbsScaler,
-    robust=RobustScaler,
-)
-
-# All available encoding strategies
-ENCODING_STRATS = dict(
-    backwarddifference=BackwardDifferenceEncoder,
-    basen=BaseNEncoder,
-    binary=BinaryEncoder,
-    catboost=CatBoostEncoder,
-    # hashing=HashingEncoder,
-    helmert=HelmertEncoder,
-    jamesstein=JamesSteinEncoder,
-    leaveoneout=LeaveOneOutEncoder,
-    mestimate=MEstimateEncoder,
-    # onehot=OneHotEncoder,
-    ordinal=OrdinalEncoder,
-    polynomial=PolynomialEncoder,
-    sum=SumEncoder,
-    target=TargetEncoder,
-    woe=WOEEncoder,
-)
-
-# All available pruning strategies
-PRUNING_STRATS = dict(
-    iforest=IsolationForest,
-    ee=EllipticEnvelope,
-    lof=LocalOutlierFactor,
-    svm=OneClassSVM,
-    dbscan=DBSCAN,
-    optics=OPTICS,
-)
-
-# All available balancing strategies
-BALANCING_STRATS = dict(
-    # clustercentroids=ClusterCentroids,
-    condensednearestneighbour=CondensedNearestNeighbour,
-    editednearestneighborus=EditedNearestNeighbours,
-    repeatededitednearestneighbours=RepeatedEditedNearestNeighbours,
-    allknn=AllKNN,
-    instancehardnessthreshold=InstanceHardnessThreshold,
-    nearmiss=NearMiss,
-    neighbourhoodcleaningrule=NeighbourhoodCleaningRule,
-    onesidedselection=OneSidedSelection,
-    randomundersampler=RandomUnderSampler,
-    tomeklinks=TomekLinks,
-    randomoversampler=RandomOverSampler,
-    smote=SMOTE,
-    smotenc=SMOTENC,
-    smoten=SMOTEN,
-    adasyn=ADASYN,
-    borderlinesmote=BorderlineSMOTE,
-    kmeanssmote=KMeansSMOTE,
-    svmsmote=SVMSMOTE,
-    smoteenn=SMOTEENN,
-    smotetomek=SMOTETomek,
-)
-
 
 # Functions ======================================================== >>
 
@@ -421,7 +302,7 @@ def to_df(data, index=None, columns=None, dtypes=None):
     if not isinstance(data, pd.DataFrame) and data is not None:
         # Assign default column names (dict already has column names)
         if not isinstance(data, dict) and columns is None:
-            columns = [f"feature {str(i)}" for i in range(1, n_cols(data) + 1)]
+            columns = [f"x{str(i)}" for i in range(n_cols(data))]
 
         # Create dataframe from sparse matrix or directly from data
         if sparse.issparse(data):
@@ -616,7 +497,7 @@ def create_acronym(fullname):
         Created acronym.
 
     """
-    from .models import MODELS
+    from atom.models import MODELS
 
     acronym = "".join([c for c in fullname if c.isupper()])
     if len(acronym) < 2 or acronym.lower() in MODELS:
@@ -645,7 +526,7 @@ def names_from_estimator(cls, est):
         Model's complete name.
 
     """
-    from .models import MODELS
+    from atom.models import MODELS
 
     for key, value in MODELS.items():
         model = value(cls, fast_intialization=True)
@@ -784,7 +665,7 @@ def partial_dependence(estimator, X, features):
         features (where the partial dependence is evaluated), and
         also to generate values for the complement features.
 
-    features : int, str or sequence
+    features: int or sequence
         The feature or pair of interacting features for which the
         partial dependency should be computed.
 
@@ -895,9 +776,9 @@ def name_cols(array, original_df, col_names):
             temp_cols.append(mask[mask].index.values[0])
         else:
             # If the column is new, use a default name
-            counter = 1
+            counter = 0
             while True:
-                n = f"feature {i + counter + original_df.shape[1] - len(col_names)}"
+                n = f"x{i + counter + original_df.shape[1] - len(col_names)}"
                 if (n not in original_df or n in col_names) and n not in temp_cols:
                     temp_cols.append(n)
                     break
@@ -927,33 +808,50 @@ def reorder_cols(df, original_df, col_names):
 
     """
     # Check if columns returned by the transformer are already in the dataset
-    for col in df.columns:
-        if col in original_df.columns and col not in col_names:
-            raise RuntimeError(
+    for col in df:
+        if col in original_df and col not in col_names:
+            raise ValueError(
                 f"Column '{col}' returned by the transformer "
                 "already exists in the original dataset."
             )
 
-    temp_df = pd.DataFrame(index=df.index)
-    for col in dict.fromkeys(list(original_df.columns) + list(df.columns)):
-        if col in df.columns:
-            temp_df[col] = df[col]
-        elif col not in col_names:
-            if len(df) != len(original_df):
-                raise ValueError(
-                    f"Length of values ({len(df)}) does not match length of index "
-                    f"({len(original_df)}). This usually happens when transformations "
-                    "that drop rows aren't applied on all the columns."
-                )
+    # Force new indices on old dataset for merge
+    try:
+        original_df.index = df.index
+    except ValueError:  # Length mismatch
+        raise IndexError(
+            f"Length of values ({len(df)}) does not match length of "
+            f"index ({len(original_df)}). This usually happens when "
+            "transformations that drop rows aren't applied on all "
+            "the columns."
+        )
 
-            temp_df[col] = original_df[col].values  # Take values to adapt to new index
+    # Define new column order
+    columns = []
+    for col in original_df:
+        if col in df or col not in col_names:
+            columns.append(col)
 
-        # Derivative cols are added after original (e.g. for one-hot encoding)
-        for col_derivative in df.columns:
-            if col_derivative.startswith(f"{col}_"):
-                temp_df[col_derivative] = df[col_derivative]
+        # Add all derivative columns: columns that originate from another
+        # and start with its progenitor name, e.g. one-hot encoded columns
+        columns.extend(
+            [c for c in df.columns if c.startswith(f"{col}_") and c not in original_df]
+        )
 
-    return temp_df
+    # Add remaining new columns (non-derivatives)
+    columns.extend([col for col in df if col not in columns])
+
+    # Merge the new and old datasets keeping the newest columns
+    new_df = df.merge(
+        right=original_df[[col for col in original_df if col in columns]],
+        how="outer",
+        left_index=True,
+        right_index=True,
+        suffixes=("", "__drop__"),
+    )
+    new_df = new_df.drop(new_df.filter(regex="__drop__$").columns, axis=1)
+
+    return new_df[columns]
 
 
 def fit_one(transformer, X=None, y=None, message=None, **fit_params):
@@ -973,7 +871,7 @@ def fit_one(transformer, X=None, y=None, message=None, **fit_params):
             transformer.fit(*args, **fit_params)
 
 
-def transform_one(transformer, X=None, y=None):
+def transform_one(transformer, X=None, y=None, method="transform"):
     """Transform the data using one estimator."""
 
     def prepare_df(out):
@@ -982,7 +880,14 @@ def transform_one(transformer, X=None, y=None):
 
         # Convert to pandas and assign proper column names
         if not isinstance(out, pd.DataFrame):
-            out = to_df(out, index=X.index, columns=name_cols(out, X, use_cols))
+            if hasattr(transformer, "get_feature_names"):
+                columns = transformer.get_feature_names()
+            elif hasattr(transformer, "get_feature_names_out"):
+                columns = transformer.get_feature_names_out()
+            else:
+                columns = name_cols(out, X, use_cols)
+
+            out = to_df(out, index=X.index, columns=columns)
 
         # Reorder columns if only a subset was used
         if len(use_cols) != X.shape[1]:
@@ -994,7 +899,7 @@ def transform_one(transformer, X=None, y=None):
     y = to_series(y, index=getattr(X, "index", None))
 
     args = []
-    transform_params = signature(transformer.transform).parameters
+    transform_params = signature(getattr(transformer, method)).parameters
     if "X" in transform_params:
         if X is not None:
             inc, exc = getattr(transformer, "_cols", (list(X.columns), None))
@@ -1006,7 +911,7 @@ def transform_one(transformer, X=None, y=None):
             args.append(y)
         elif "X" not in transform_params:
             return X, y  # If y is None and needed, and no X in transformer, skip it
-    output = transformer.transform(*args)
+    output = getattr(transformer, method)(*args)
 
     # Transform can return X, y or both
     if isinstance(output, tuple):
@@ -1261,11 +1166,6 @@ def false_negative_rate(y_true, y_pred):
     return float(fn / (fn + tp))
 
 
-def lift(y_true, y_pred):
-    tn, fp, fn, tp = confusion_matrix(y_true, y_pred).ravel()
-    return float((tp / (tp + fp)) / ((tp + fn) / (tp + tn + fp + fn)))
-
-
 # Scorers not predefined by sklearn
 CUSTOM_SCORERS = dict(
     tn=true_negatives,
@@ -1276,7 +1176,6 @@ CUSTOM_SCORERS = dict(
     tpr=true_positive_rate,
     tnr=true_negative_rate,
     fnr=false_negative_rate,
-    lift=lift,
     mcc=matthews_corrcoef,
 )
 
@@ -1506,6 +1405,7 @@ class ShapExplanation:
                 self._explainer = Explainer(self.T.estimator, self.T.X_train)
             except Exception:
                 # Prediction attr to use (predict_proba > decision_function > predict)
+                # If method is provided as first arg, selects always Permutation
                 attr = getattr(self.T.estimator, get_proba_attr(self.T))
                 self._explainer = Explainer(attr, self.T.X_train)
 
@@ -1538,8 +1438,13 @@ class ShapExplanation:
         # Get rows that still need to be calculated
         calculate = df.loc[[i for i in df.index if i not in self._shap_values.index]]
         if not calculate.empty:
-            # Additivity check fails sometimes for no apparent reason
             kwargs = {}
+
+            # Minimum of 2 * n_features + 1 evals required (default=500)
+            if "max_evals" in signature(self.explainer.__call__).parameters:
+                kwargs["max_evals"] = 2 * self.T.n_features + 1
+
+            # Additivity check fails sometimes for no apparent reason
             if "check_additivity" in signature(self.explainer.__call__).parameters:
                 kwargs["check_additivity"] = False
 
@@ -1550,22 +1455,29 @@ class ShapExplanation:
             for i, idx in enumerate(calculate.index):
                 self._shap_values.loc[idx] = self._explanation.values[i]
 
+        # Don't use attribute to not save plot-specific changes
+        explanation = copy(self._explanation)
+
         # Update the explanation object
-        self._explanation.values = np.stack(self._shap_values.loc[df.index].values)
-        if isinstance(self._explanation.base_values, SEQUENCE):
-            self._explanation.base_values = self._explanation.base_values[0]
-        self._explanation.data = self.T.X.loc[df.index, :].to_numpy()
+        explanation.values = np.stack(self._shap_values.loc[df.index].values)
+        explanation.base_values = explanation.base_values[0]
+        explanation.data = self.T.X.loc[df.index, :].to_numpy()
 
         # Select the target values from the array
-        if self._explanation.values.ndim > 2:
-            self._explanation = self._explanation[:, :, target]
-        if only_one:  # Rows should be a df with one row only
-            self._explanation = self._explanation[0]
+        if explanation.values.ndim > 2:
+            explanation.values = explanation.values[:, :, target]
+        if only_one:  # Attributes should be 1-dimensional
+            explanation.values = explanation.values[0]
+            explanation.data = explanation.data[0]
+
+            # For some models like LGB it's a scalar already
+            if isinstance(explanation.base_values, np.ndarray):
+                explanation.base_values = explanation.base_values[target]
 
         if feature is None:
-            return self._explanation
+            return explanation
         else:
-            return self._explanation[:, feature]
+            return explanation[:, feature]
 
     def get_shap_values(self, df, target=1, return_all_classes=False):
         """Get shap values from the Explanation object."""
@@ -1592,11 +1504,9 @@ class ShapExplanation:
                     getattr(self.T, f"{get_proba_attr(self.T)}_train")
                 )
 
-        if return_one and isinstance(self._expected_value, (list, np.ndarray)):
+        if return_one and isinstance(self._expected_value, SEQUENCE):
             if len(self._expected_value) == self.T.y.nunique():
                 return self._expected_value[target]  # Return target expected value
-        if not return_one and not isinstance(self._expected_value, (list, np.ndarray)):
-            return [1 - self._expected_value, self._expected_value]  # Must be binary
 
         return self._expected_value
 
@@ -1620,9 +1530,12 @@ class CustomDict(MutableMapping):
         return key.lower() if isinstance(key, str) else key
 
     def _get_key(self, key):
-        for k in self.__keys:
-            if self._conv(k) == self._conv(key):
-                return k
+        if isinstance(key, (int, np.integer)) and key not in self.__keys:
+            return self.__keys[key]
+        else:
+            for k in self.__keys:
+                if self._conv(k) == self._conv(key):
+                    return k
 
         raise KeyError(key)
 
@@ -1634,8 +1547,8 @@ class CustomDict(MutableMapping):
         unless you want the order to be arbitrary.
 
         """
-        self.__keys = []
-        self.__data = {}
+        self.__keys = []  # States the order
+        self.__data = {}  # Contains the values
 
         if iterable_or_mapping is not None:
             try:
@@ -1668,7 +1581,8 @@ class CustomDict(MutableMapping):
         self.__data[self._conv(key)] = value
 
     def __delitem__(self, key):
-        self.__keys.remove(self._get_key(key))
+        key = self._get_key(key)
+        self.__keys.remove(key)
         del self.__data[self._conv(key)]
 
     def __iter__(self):
