@@ -7,36 +7,33 @@ Description: Unit tests for atom.py
 
 """
 
-# Standard packages
 import glob
+from unittest.mock import patch
 
+import numpy as np
 import pandas as pd
 import pytest
-import numpy as np
-from unittest.mock import patch
-from sklearn.metrics import get_scorer
+from category_encoders.leave_one_out import LeaveOneOutEncoder
 from sklearn.decomposition import PCA
+from sklearn.ensemble import RandomForestClassifier
 from sklearn.feature_selection import SelectFromModel
 from sklearn.kernel_approximation import RBFSampler
 from sklearn.linear_model import LassoLarsCV
-from sklearn.ensemble import RandomForestClassifier
-from sklearn.pipeline import Pipeline
+from sklearn.metrics import get_scorer
 from sklearn.neural_network import MLPClassifier
+from sklearn.pipeline import Pipeline
 from sklearn.preprocessing import (
-    LabelEncoder,
-    OneHotEncoder,
-    StandardScaler,
-    RobustScaler,
+    LabelEncoder, OneHotEncoder, RobustScaler, StandardScaler,
 )
 
-# Own modules
 from atom import ATOMClassifier, ATOMRegressor
-from atom.data_cleaning import Scaler, Pruner
+from atom.data_cleaning import Pruner, Scaler
 from atom.utils import check_scaling
-from .utils import (
-    FILE_DIR, X_bin, y_bin, X_class, y_class, X_reg, y_reg, X_sparse,
-    X_text, X10, X10_nan, X10_str, X10_str2, X10_dt, y10, y10_str,
-    y10_sn, X20_out,
+
+from .conftest import (
+    X10, DummyTransformer, X10_dt, X10_nan, X10_str, X10_str2, X20_out, X_bin,
+    X_class, X_reg, X_sparse, X_text, y10, y10_sn, y10_str, y_bin, y_class,
+    y_reg,
 )
 
 
@@ -56,8 +53,8 @@ def test_task_assignment():
 
 def test_raise_one_target_value():
     """Assert that error raises when there is only 1 target value."""
-    y = [1 for _ in range(len(y_bin))]  # All targets are equal to 1
-    pytest.raises(ValueError, ATOMClassifier, X_bin, y, random_state=1)
+    with pytest.raises(ValueError, match=r".*1 target value.*"):
+        ATOMClassifier(X_bin, [1 for _ in range(len(y_bin))], random_state=1)
 
 
 # Test magic methods =============================================== >>
@@ -173,7 +170,7 @@ def test_n_nans():
 def test_numerical():
     """Assert that numerical returns the names of the numerical columns."""
     atom = ATOMClassifier(X10_str, y10, random_state=1)
-    assert atom.numerical == ["feature 1", "feature 2", "feature 4"]
+    assert len(atom.numerical) == 3
 
 
 def test_n_numerical():
@@ -185,7 +182,7 @@ def test_n_numerical():
 def test_categorical():
     """Assert that categorical returns the names of categorical columns."""
     atom = ATOMClassifier(X10_str, y10, random_state=1)
-    assert atom.categorical == ["feature 3"]
+    assert len(atom.categorical) == 1
 
 
 def test_n_categorical():
@@ -272,7 +269,7 @@ def test_automl_invalid_scoring():
 
 
 @pytest.mark.parametrize("distributions", [None, "norm", ["norm", "pearson3"]])
-@pytest.mark.parametrize("columns", ["feature 1", 1, None])
+@pytest.mark.parametrize("columns", ["x0", 1, None])
 def test_distribution(distributions, columns):
     """Assert that the distribution method and file are created."""
     atom = ATOMClassifier(X10_str, y10, random_state=1)
@@ -333,7 +330,7 @@ def test_export_pipeline_memory(func):
 def test_report(cls):
     """Assert that the report method and file are created."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
-    atom.report(n_rows=10, filename="report")
+    atom.report(filename="report")
     cls.return_value.to_file.assert_called_once_with("report.html")
 
 
@@ -346,14 +343,14 @@ def test_reset():
     atom.run("LR")
     atom.reset()
     assert not atom.models and len(atom._branches) == 1
-    assert atom["feature 3"].dtype.name == "object"  # Is reset back to str
+    assert atom["x2"].dtype.name == "object"  # Is reset back to str
 
 
 def test_save_data():
     """Assert that the dataset is saved to a csv file."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
-    atom.save_data(FILE_DIR + "auto")
-    assert glob.glob(FILE_DIR + "ATOMClassifier_dataset.csv")
+    atom.save_data("auto")
+    assert glob.glob("ATOMClassifier_dataset.csv")
 
 
 def test_shrink_dtypes_excluded():
@@ -437,7 +434,7 @@ def test_transform_method():
     """ Assert that the transform method works as intended."""
     atom = ATOMClassifier(X10_str, y10, random_state=1)
     atom.encode(max_onehot=None)
-    assert atom.transform(X10_str)["feature 3"].dtype.kind in "ifu"
+    assert atom.transform(X10_str)["x2"].dtype.kind in "ifu"
 
 
 def test_transform_not_train_only():
@@ -535,8 +532,8 @@ def test_add_transformer_only_y():
 def test_returned_column_already_exists():
     """Assert that an error is raised if an existing column is returned."""
     atom = ATOMClassifier(X_text, y10, random_state=1)
-    atom.apply(lambda x: 1, columns="new")
-    with pytest.raises(RuntimeError, match=r".*already exists in the original.*"):
+    atom.apply(lambda x: 1, columns="corpus_new")
+    with pytest.raises(ValueError, match=r".*already exists in the original.*"):
         atom.vectorize(columns="corpus")
 
 
@@ -549,31 +546,41 @@ def test_add_sparse_matrices():
 
 def test_add_keep_column_names():
     """Assert that the column names are kept after transforming."""
-    atom = ATOMClassifier(X_bin, y_bin, random_state=1)
+    atom = ATOMClassifier(X10_str, y10, random_state=1)
 
-    # When the columns are only transformed
-    cols = atom.columns.copy()
+    # Transformer has method get_feature_names
+    atom.add(LeaveOneOutEncoder(return_df=False))
+    assert atom.features.tolist() == ["x0", "x1", "x2", "x3"]
+
+    # Transformer has method get_feature_names_out
     atom.add(StandardScaler())
-    assert atom.columns == cols
+    assert atom.features.tolist() == ["x0", "x1", "x2", "x3"]
 
-    # When columns were removed
-    cols = atom.columns.copy()
-    atom.add(SelectFromModel(RandomForestClassifier()))
-    assert all(col in cols for col in atom.columns)
+    # Transformer keeps rows equal
+    atom.add(DummyTransformer(strategy="equal"))
+    assert atom.features.tolist() == ["x0", "x1", "x2", "x3"]
+
+    # Transformer drops rows
+    atom.add(DummyTransformer(strategy="drop"))
+    assert atom.features.tolist() == ["x0", "x2", "x3"]
+
+    # Transformer adds rows
+    atom.add(DummyTransformer(strategy="add"), columns="!x2")
+    assert atom.features.tolist() == ["x0", "x2", "x3", "x4"]
 
 
 def test_raise_length_mismatch():
     """Assert that an error is raised when there's a mismatch in row length."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
-    with pytest.raises(ValueError, match=r".*does not match length.*"):
+    with pytest.raises(IndexError, match=r".*does not match length.*"):
         atom.prune(columns=[2, 4])
 
 
 def test_add_derivative_columns_keep_position():
     """Assert that derivative columns go after the original."""
     atom = ATOMClassifier(X10_str, y10, random_state=1)
-    atom.encode(columns="feature 3")
-    assert atom.columns[2:5] == ["feature 3_a", "feature 3_b", "feature 3_c"]
+    atom.encode(columns="x2")
+    assert list(atom.columns[2:5]) == ["x2_a", "x2_b", "x2_c"]
 
 
 def test_add_sets_are_kept_equal():
@@ -623,7 +630,7 @@ def test_apply_same_column():
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.apply(lambda x: 1, columns=0)
     assert atom["mean radius"].sum() == atom.shape[0]
-    assert str(atom.pipeline[0]).startswith("FuncTransformer(func=<lambda>")
+    assert str(atom.pipeline[0]).startswith("FunctionTransformer(func=<lambda>")
 
 
 def test_apply_new_column():
@@ -662,11 +669,11 @@ def test_scale():
     assert hasattr(atom, "standard")
 
 
-def test_gauss():
-    """Assert that the gauss method transforms the features."""
+def test_normalize():
+    """Assert that the normalize method transforms the features."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     X = atom.X.copy()
-    atom.gauss()
+    atom.normalize()
     assert not atom.X.equals(X)
     assert hasattr(atom, "yeojohnson")
 
@@ -741,10 +748,10 @@ def test_tokenize():
     assert atom["corpus"][0] == ["I", "àm", "in", "ne", "'", "w", "york"]
 
 
-def test_normalize():
-    """Assert that the normalize method normalizes the corpus."""
+def test_textnormalize():
+    """Assert that the textnormalize method normalizes the corpus."""
     atom = ATOMClassifier(X_text, y10, shuffle=False, random_state=1)
-    atom.normalize(stopwords=False, custom_stopwords=["yes"])
+    atom.textnormalize(stopwords=False, custom_stopwords=["yes"])
     assert atom["corpus"][0] == ["I", "àm", "in", "ne'w", "york"]
 
 
@@ -776,8 +783,8 @@ def test_feature_generation():
 def test_feature_generation_attributes():
     """Assert that the attrs from feature_generation are passed to atom."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
-    atom.feature_generation("GFG", n_features=2)
-    assert hasattr(atom, "symbolic_transformer")
+    atom.feature_generation("gfg", n_features=2)
+    assert hasattr(atom, "gfg")
     assert hasattr(atom, "genetic_features")
 
 
@@ -794,12 +801,12 @@ def test_default_solver_univariate():
     # For classification tasks
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.feature_selection(strategy="univariate", solver=None, n_features=8)
-    assert atom.pipeline[0]._solver.__name__ == "f_classif"
+    assert atom.pipeline[0].univariate.score_func.__name__ == "f_classif"
 
     # For regression tasks
     atom = ATOMRegressor(X_reg, y_reg, random_state=1)
     atom.feature_selection(strategy="univariate", solver=None, n_features=8)
-    assert atom.pipeline[0]._solver.__name__ == "f_regression"
+    assert atom.pipeline[0].univariate.score_func.__name__ == "f_regression"
 
 
 def test_winner_solver_after_run():
@@ -807,8 +814,8 @@ def test_winner_solver_after_run():
     atom = ATOMClassifier(X_class, y_class, random_state=1)
     atom.run("LR")
     atom.branch = "fs_branch"
-    atom.feature_selection(strategy="SFM", solver=None, n_features=8)
-    assert atom.pipeline[0]._solver is atom.winner.estimator
+    atom.feature_selection(strategy="sfm", solver=None, n_features=8)
+    assert atom.pipeline[0].sfm.estimator_ is atom.winner.estimator
 
 
 def test_default_solver_from_task():
@@ -816,12 +823,12 @@ def test_default_solver_from_task():
     # For classification tasks
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.feature_selection(strategy="rfe", solver="lgb", n_features=8)
-    assert atom.pipeline[0]._solver.__class__.__name__ == "LGBMClassifier"
+    assert atom.pipeline[0].rfe.estimator_.__class__.__name__ == "LGBMClassifier"
 
     # For regression tasks
     atom = ATOMRegressor(X_reg, y_reg, random_state=1)
     atom.feature_selection(strategy="rfe", solver="lgb", n_features=25)
-    assert atom.pipeline[0]._solver.__class__.__name__ == "LGBMRegressor"
+    assert atom.pipeline[0].rfe.estimator_.__class__.__name__ == "LGBMRegressor"
 
 
 @patch("atom.feature_engineering.SequentialFeatureSelector")
@@ -856,7 +863,7 @@ def test_scaling_is_passed():
     atom = ATOMRegressor(X_reg, y_reg, random_state=1)
     atom.scale("minmax")
     atom.run("LGB")
-    assert atom.dataset.equals(atom.lgb.dataset)
+    pd.testing.assert_frame_equal(atom.dataset, atom.lgb.dataset)
 
 
 def test_errors_are_updated():
@@ -872,6 +879,14 @@ def test_errors_are_updated():
     assert atom.models == "LGB"
 
 
+def test_models_are_replaced():
+    """Assert that models with the same name are replaced."""
+    atom = ATOMRegressor(X_reg, y_reg, random_state=1)
+    atom.run(["OLS", "Tree"])
+    atom.run("OLS")
+    assert atom.models == ["Tree", "OLS"]
+
+
 def test_models_and_metric_are_updated():
     """Assert that the models and metric attributes are updated correctly."""
     atom = ATOMRegressor(X_reg, y_reg, random_state=1)
