@@ -7,23 +7,22 @@ Description: Unit tests for basemodel.py
 
 """
 
-# Standard packages
 import glob
-import pytest
+from unittest.mock import patch
+
 import numpy as np
 import pandas as pd
-from unittest.mock import patch
-from skopt.space.space import Integer
-from skopt.learning import GaussianProcessRegressor
+import pytest
 from sklearn.calibration import CalibratedClassifierCV
 from sklearn.metrics import accuracy_score, r2_score, recall_score
+from skopt.learning import GaussianProcessRegressor
+from skopt.space.space import Integer
 
-# Own modules
 from atom import ATOMClassifier, ATOMRegressor
 from atom.utils import check_scaling
-from .utils import (
-    FILE_DIR, X_bin, y_bin, X_class, y_class, X_reg, y_reg,
-    X_idx, y_idx, X10_str, y10,
+
+from .conftest import (
+    X10_str, X_bin, X_class, X_idx, X_reg, y10, y_bin, y_class, y_idx, y_reg,
 )
 
 
@@ -66,9 +65,7 @@ def test_getitem():
     """Assert that the models are subscriptable."""
     atom = ATOMClassifier(X_class, y_class, random_state=1)
     atom.run("Tree")
-    print(atom.dataset["alcohol"])
-    print(atom.tree["alcohol"])
-    assert atom.tree["alcohol"].equals(atom.dataset["alcohol"])
+    pd.testing.assert_series_equal(atom.tree["alcohol"], atom.dataset["alcohol"])
     assert isinstance(atom.tree[["alcohol", "ash"]], pd.DataFrame)
     with pytest.raises(TypeError, match=r".*subscriptable with types.*"):
         print(atom.tree[2])
@@ -175,7 +172,7 @@ def test_all_base_estimators(est):
     atom.run("LR", n_calls=5, bo_params={"base_estimator": est})
 
 
-def test_sample_weights_fit():
+def test_sample_weight_fit():
     """Assert that sample weights can be used with the BO."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.run(
@@ -197,7 +194,12 @@ def test_bo_with_pipeline():
 def test_early_stopping(model):
     """Assert than early stopping works."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
-    atom.run(model, n_calls=5, bo_params={"early_stopping": 0.1})
+    atom.run(
+        models=model,
+        n_calls=5,
+        est_params={"n_estimators": 10},
+        bo_params={"early_stopping": 0.1},
+    )
     assert getattr(atom, model)._stopped != ("---", "---")
 
 
@@ -373,7 +375,7 @@ def test_score_custom_metric():
     assert atom.tree.score(X_bin, y_bin, metric="recall") == recall
 
 
-def test_score_with_sample_weights():
+def test_score_with_sample_weight():
     """Assert that the score method works when sample weights are provided."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.run("Tree")
@@ -399,7 +401,7 @@ def test_dataset_property():
     """Assert that the dataset property returns scaled data if needed."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.run(["MNB", "LR"])
-    assert atom.dataset.equals(atom.mnb.dataset)
+    pd.testing.assert_frame_equal(atom.dataset, atom.mnb.dataset)
     assert check_scaling(atom.lr.dataset)
 
 
@@ -407,7 +409,7 @@ def test_train_property():
     """Assert that the train property returns scaled data if needed."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.run(["MNB", "LR"])
-    assert atom.train.equals(atom.mnb.train)
+    pd.testing.assert_frame_equal(atom.train, atom.mnb.train)
     assert check_scaling(atom.lr.train)
 
 
@@ -415,7 +417,7 @@ def test_test_property():
     """Assert that the test property returns scaled data if needed."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.run(["MNB", "LR"])
-    assert atom.test.equals(atom.mnb.test)
+    pd.testing.assert_frame_equal(atom.test, atom.mnb.test)
     assert check_scaling(atom.lr.test)
 
 
@@ -432,7 +434,7 @@ def test_X_property():
     """Assert that the X property returns scaled data if needed."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.run(["MNB", "LR"])
-    assert atom.X.equals(atom.mnb.X)
+    pd.testing.assert_frame_equal(atom.X, atom.mnb.X)
     assert check_scaling(atom.lr.X)
 
 
@@ -440,15 +442,15 @@ def test_y_property():
     """Assert that the y property is returned unchanged."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.run(["MNB", "LR"])
-    assert atom.y.equals(atom.mnb.y)
-    assert atom.y.equals(atom.lr.y)
+    pd.testing.assert_series_equal(atom.y, atom.mnb.y)
+    pd.testing.assert_series_equal(atom.y, atom.lr.y)
 
 
 def test_X_train_property():
     """Assert that the X_train property returns scaled data if needed."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.run(["MNB", "LR"])
-    assert atom.X_train.equals(atom.mnb.X_train)
+    pd.testing.assert_frame_equal(atom.X_train, atom.mnb.X_train)
     assert check_scaling(atom.lr.X_train)
 
 
@@ -456,7 +458,7 @@ def test_X_test_property():
     """Assert that the X_test property returns scaled data if needed."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.run(["MNB", "LR"])
-    assert atom.X_test.equals(atom.mnb.X_test)
+    pd.testing.assert_frame_equal(atom.X_test, atom.mnb.X_test)
     assert check_scaling(atom.lr.X_test)
 
 
@@ -464,22 +466,22 @@ def test_X_holdout_property():
     """Assert that the X_holdout property is calculated."""
     atom = ATOMClassifier(X_bin, y_bin, holdout_size=0.1, random_state=1)
     atom.run("MNB")
-    assert atom.mnb.X_holdout.equals(atom.mnb.holdout.iloc[:, :-1])
+    pd.testing.assert_frame_equal(atom.mnb.X_holdout, atom.mnb.holdout.iloc[:, :-1])
 
 
 def test_y_train_property():
     """Assert that the y_train property is returned unchanged."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.run(["MNB", "LR"])
-    assert atom.y_train.equals(atom.mnb.y_train)
-    assert atom.y_train.equals(atom.lr.y_train)
+    pd.testing.assert_series_equal(atom.y_train, atom.mnb.y_train)
+    pd.testing.assert_series_equal(atom.y_train, atom.lr.y_train)
 
 
 def test_y_holdout_property():
     """Assert that the y_holdout property is calculated."""
     atom = ATOMClassifier(X_bin, y_bin, holdout_size=0.1, random_state=1)
     atom.run("MNB")
-    assert atom.mnb.y_holdout.equals(atom.mnb.holdout.iloc[:, -1])
+    pd.testing.assert_series_equal(atom.mnb.y_holdout, atom.mnb.holdout.iloc[:, -1])
 
 
 # Test utility methods ============================================= >>
@@ -544,51 +546,50 @@ def test_cross_validate():
     """Assert that the cross_validate method works as intended."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.run("LR")
-    assert isinstance(atom.lr.cross_validate(), dict)
-    assert isinstance(atom.lr.cross_validate(scoring="AP"), dict)
+    assert isinstance(atom.lr.cross_validate(), pd.DataFrame)
+    assert isinstance(atom.lr.cross_validate(scoring="AP"), pd.DataFrame)
 
 
 def test_dashboard_dataset_no_holdout():
     """Assert that an error is raised when there's no holdout set."""
     atom = ATOMRegressor(X_reg, y_reg, random_state=1)
-    atom.run("RF")
+    atom.run("Tree")
     with pytest.raises(ValueError, match=r".*No holdout data set.*"):
-        atom.rf.dashboard(dataset="holdout")
+        atom.tree.dashboard(dataset="holdout")
 
 
 def test_dashboard_invalid_dataset():
     """Assert that an error is raised when dataset is invalid."""
     atom = ATOMRegressor(X_reg, y_reg, random_state=1)
-    atom.run("RF")
+    atom.run("Tree")
     with pytest.raises(ValueError, match=r".*dataset parameter.*"):
-        atom.rf.dashboard(dataset="invalid")
+        atom.tree.dashboard(dataset="invalid")
 
 
-@patch("explainerdashboard.ExplainerDashboard.run")
+@patch("explainerdashboard.ExplainerDashboard")
 @pytest.mark.parametrize("dataset", ["train", "both", "holdout"])
-def test_dashboard(func, dataset):
-    """Assert that an error is raised when dataset is invalid."""
+def test_dashboard_classification(func, dataset):
+    """Assert that the dashboard method calls the underlying package."""
     atom = ATOMClassifier(X_bin, y_bin, holdout_size=0.1, random_state=1)
-    atom.run("RF")
-    atom.rf.dashboard(dataset=dataset)
+    atom.run("Tree")
+    atom.tree.dashboard(dataset=dataset, filename="dashboard")
     func.assert_called_once()
 
 
-@patch("explainerdashboard.ExplainerDashboard.run")
-def test_dashboard_is_saved(func):
-    """Assert that an error is raised when dataset is invalid."""
+@patch("explainerdashboard.ExplainerDashboard")
+def test_dashboard_regression(func):
+    """Assert that the dashboard method calls the underlying package."""
     atom = ATOMRegressor(X_reg, y_reg, holdout_size=0.1, random_state=1)
-    atom.run("RF")
-    atom.rf.dashboard(filename=FILE_DIR + "dashboard")
+    atom.run("Tree")
+    atom.tree.dashboard()
     func.assert_called_once()
-    assert glob.glob(FILE_DIR + "dashboard.html")
 
 
 def test_delete():
     """Assert that models can be deleted."""
     atom = ATOMRegressor(X_reg, y_reg, random_state=1)
-    atom.run("RF")
-    atom.rf.delete()
+    atom.run("Tree")
+    atom.tree.delete()
     assert not atom.models
     assert not atom.metric
 
@@ -679,6 +680,14 @@ def test_export_pipeline_memory(func):
     func.assert_called_once()
 
 
+def test_full_train_no_holdout():
+    """Assert that an error is raised when include_holdout=True with no set."""
+    atom = ATOMClassifier(X_bin, y_bin, random_state=1)
+    atom.run("LGB")
+    with pytest.raises(ValueError, match=r".*no holdout data set.*"):
+        atom.lgb.full_train(include_holdout=True)
+
+
 def test_full_train():
     """Assert that the full_train method trains on the test set."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
@@ -738,8 +747,8 @@ def test_save_estimator():
     """Assert that the save_estimator saves a pickle file."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.run("MNB")
-    atom.mnb.save_estimator(FILE_DIR + "auto")
-    assert glob.glob(FILE_DIR + "MultinomialNB")
+    atom.mnb.save_estimator("auto")
+    assert glob.glob("MultinomialNB")
 
 
 def test_transform():
