@@ -7,20 +7,19 @@ Description: Unit tests for basepredictor.py
 
 """
 
-# Standard packages
-import pytest
 import numpy as np
 import pandas as pd
+import pytest
 from sklearn.discriminant_analysis import LinearDiscriminantAnalysis as LDA
 
-# Own modules
 from atom import ATOMClassifier, ATOMRegressor
 from atom.branch import Branch
 from atom.training import DirectClassifier
 from atom.utils import NotFittedError, merge
-from .utils import (
-    X_bin, y_bin, X_class, y_class, X_reg, y_reg, X_idx, y_idx,
-    bin_train, bin_test, X10, y10, X10_str,
+
+from .conftest import (
+    X10, X10_str, X_bin, X_class, X_idx, X_reg, bin_test, bin_train, y10,
+    y_bin, y_class, y_idx, y_reg,
 )
 
 
@@ -225,6 +224,13 @@ def test_errors_property():
     assert "LGB" in atom.errors
 
 
+def test_winners_property():
+    """Assert that the winners property returns the best models."""
+    atom = ATOMClassifier(X_bin, y_bin, random_state=1)
+    atom.run(["LR", "Tree", "LGB"], n_calls=0)
+    assert atom.winners == ["LR", "LGB", "Tree"]
+
+
 def test_winner_property():
     """Assert that the winner property returns the best model in the pipeline."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
@@ -304,7 +310,7 @@ def test_score():
     assert isinstance(atom.score(X_bin, y_bin), float)
 
 
-def test_score_sample_weights():
+def test_score_sample_weight():
     """Assert that the score method works with sample weights."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     atom.run("LR")
@@ -507,18 +513,18 @@ def test_available_models():
     assert "BR" not in models["acronym"].unique()
 
 
-def test_clear():
-    """Assert that the clear method resets all model's attributes."""
-    atom = ATOMClassifier(X_bin, y_bin, random_state=1)
-    atom.run(["LR", "LGB"])
-    atom.lgb.beeswarm_plot()
-    assert atom.lr._pred[3] is not None
-    assert atom.lr._scores["train"]
-    assert not atom.lgb._shap._shap_values.empty
-    atom.clear()
-    assert atom.lr._pred == [None] * 15
-    assert not atom.lr._scores["train"]
-    assert atom.lgb._shap._shap_values.empty
+# def test_clear():
+#     """Assert that the clear method resets all model's attributes."""
+#     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
+#     atom.run(["LR", "LGB"])
+#     atom.lgb.beeswarm_plot()
+#     assert atom.lr._pred[3] is not None
+#     assert atom.lr._scores["train"]
+#     assert not atom.lgb._shap._shap_values.empty
+#     atom.clear()
+#     assert atom.lr._pred == [None] * 15
+#     assert not atom.lr._scores["train"]
+#     assert atom.lgb._shap._shap_values.empty
 
 
 def test_delete_default():
@@ -700,6 +706,15 @@ def test_stacking_invalid_models():
         atom.stacking()
 
 
+def test_stacking_invalid_name():
+    """Assert that an error is raised when the model already exists."""
+    atom = ATOMClassifier(X_bin, y_bin, random_state=1)
+    atom.run(["LR", "Tree"])
+    atom.stacking()
+    with pytest.raises(ValueError, match=r".*multiple Stacking.*"):
+        atom.stacking()
+
+
 def test_stacking_custom_models():
     """Assert that stacking can be created selecting the models."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
@@ -764,6 +779,15 @@ def test_voting():
     assert atom.vote2._run
 
 
+def test_voting_invalid_name():
+    """Assert that an error is raised when the model already exists."""
+    atom = ATOMClassifier(X_bin, y_bin, random_state=1)
+    atom.run(["LR", "Tree"])
+    atom.voting()
+    with pytest.raises(ValueError, match=r".*multiple Voting.*"):
+        atom.voting()
+
+
 def test_voting_invalid_models():
     """Assert that an error is raised when <2 models."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
