@@ -7,20 +7,20 @@ Description: Unit tests for plots.py
 
 """
 
-# Standard packages
 import glob
-import pytest
 from unittest.mock import patch
-from sklearn.metrics import f1_score, get_scorer
-from sklearn.linear_model import LogisticRegression
 
-# Own modules
+import pytest
+from sklearn.linear_model import LogisticRegression
+from sklearn.metrics import f1_score, get_scorer
+
 from atom import ATOMClassifier, ATOMRegressor
 from atom.plots import BasePlotter
 from atom.utils import NotFittedError
-from .utils import (
-    FILE_DIR, X_bin, y_bin, X_class, y_class, X_reg, y_reg,
-    X_sparse, X_text, X10, X10_str, y10, y10_str,
+
+from .conftest import (
+    X10, X10_str, X_bin, X_class, X_reg, X_sparse, X_text, y10, y10_str, y_bin,
+    y_class, y_reg,
 )
 
 
@@ -178,7 +178,7 @@ def test_plot_scatter_matrix():
     atom.plot_scatter_matrix(columns=[0, 1, 2], display=False)
 
 
-@pytest.mark.parametrize("columns", [2, "feature 1", [0, 1]])
+@pytest.mark.parametrize("columns", [2, "x0", [0, 1]])
 def test_plot_distribution(columns):
     """Assert that the plot_distribution method work as intended."""
     atom = ATOMClassifier(X10_str, y10, random_state=1)
@@ -210,32 +210,21 @@ def test_plot_ngrams(ngram):
     atom.plot_ngrams(ngram=ngram, display=False)  # When corpus are tokens
 
 
-def test_plot_pipeline():
-    """Assert that the plot_pipeline method work as intended."""
-    atom = ATOMClassifier(X_bin, y_bin, random_state=1)
-    atom.impute()
-    atom.prune()
-    atom.feature_selection("univariate", n_features=10)
-    atom.successive_halving(["Tree", "LGB"])
-    atom.plot_pipeline(model=None, show_params=False, display=False)
-    atom.plot_pipeline(model="Tree", title="Pipeline plot", display=False)
-
-
 @pytest.mark.parametrize("X", [X10, X_sparse])
 def test_plot_pca(X):
     """Assert that the plot_pca method work as intended."""
     atom = ATOMClassifier(X, y10, random_state=1)
     pytest.raises(PermissionError, atom.plot_pca)
-    atom.feature_selection(strategy="PCA", n_features=2)
+    atom.feature_selection(strategy="pca", n_features=2)
     atom.plot_pca(display=False)
 
 
-@pytest.mark.parametrize("show", [10, 200, None])
+@pytest.mark.parametrize("show", [10, None])
 def test_plot_components(show):
     """Assert that the plot_components method work as intended."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     pytest.raises(PermissionError, atom.plot_components)
-    atom.feature_selection(strategy="PCA", n_features=10)
+    atom.feature_selection(strategy="pca", n_features=10)
     pytest.raises(ValueError, atom.plot_components, show=0)
     atom.plot_components(show=show, display=False)
 
@@ -247,10 +236,30 @@ def test_plot_rfecv(scoring):
     pytest.raises(PermissionError, atom.plot_rfecv)
     atom.run("lr", metric="precision")
     atom.branch = "fs_branch"
-    atom.feature_selection(strategy="RFECV", n_features=10, scoring=scoring)
+    atom.feature_selection(strategy="rfecv", n_features=10, scoring=scoring)
     atom.plot_rfecv(display=False)
 
 
+def test_plot_pipeline():
+    """Assert that the plot_pipeline method work as intended."""
+    atom = ATOMClassifier(X_bin, y_bin, random_state=1)
+    atom.run("Tree")
+    atom.plot_pipeline(display=False)  # No transformers
+
+    atom = ATOMRegressor(X_reg, y_reg, random_state=1)
+    atom.scale()
+    atom.plot_pipeline(display=False)  # No model
+
+    atom.run("Tree", n_calls=2, n_initial_points=1)
+    atom.tree.plot_pipeline(display=False)  # Only one branch
+
+    atom.branch = "b2"
+    atom.prune()
+    atom.run(["OLS", "EN"])
+    atom.voting()
+    atom.plot_pipeline(title="Pipeline plot", display=False)  # Multiple branches
+
+
 @pytest.mark.parametrize("metric", ["f1", ["f1", "recall"]])
 def test_plot_successive_halving(metric):
     """Assert that the plot_successive_halving method work as intended."""
@@ -442,10 +451,8 @@ def test_plot_permutation_importance():
     atom.lgb.plot_permutation_importance(display=False)
 
 
-@pytest.mark.parametrize("columns", [(("ash", "alcohol"), 2, "ash"), ("ash", 2), 2])
-def test_plot_partial_dependence(columns):
-    """Assert that the plot_partial_dependence method work as intended."""
-    # For binary classification tasks
+def test_plot_partial_dependence_binary():
+    """Assert that the plot_partial_dependence method work for binary tasks."""
     atom = ATOMClassifier(X_bin, y_bin, random_state=1)
     pytest.raises(NotFittedError, atom.plot_partial_dependence)
     atom.run(["Tree", "LGB"], metric="f1")
@@ -462,7 +469,7 @@ def test_plot_partial_dependence(columns):
     with pytest.raises(ValueError, match=r".*should be single or in pairs.*"):
         atom.lgb.plot_partial_dependence(columns=[(0, 1, 2), 2], display=False)
 
-    # Pair for multi-model
+    # Pair for multimodel
     with pytest.raises(ValueError, match=r".*when plotting multiple models.*"):
         atom.plot_partial_dependence(columns=[(0, 2), 2], display=False)
 
@@ -482,7 +489,10 @@ def test_plot_partial_dependence(columns):
     atom.lgb.plot_feature_importance(show=5, display=False)
     atom.lgb.plot_partial_dependence(display=False)
 
-    # For multiclass classification tasks
+
+@pytest.mark.parametrize("columns", [(("ash", "alcohol"), 2, "ash"), ("ash", 2), 2])
+def test_plot_partial_dependence_multiclass(columns):
+    """Assert that the plot_partial_dependence method work for multiclass tasks."""
     atom = ATOMClassifier(X_class, y_class, random_state=1)
     atom.run(["Tree", "LGB"], metric="f1_macro")
 
@@ -611,8 +621,8 @@ def test_force_plot():
     # Own calculation of expected value
     atom.mlp.force_plot(index=100, matplotlib=True, display=False)
 
-    atom.lr.force_plot(matplotlib=False, filename=FILE_DIR + "force", display=True)
-    assert glob.glob(FILE_DIR + "force.html")
+    atom.lr.force_plot(matplotlib=False, filename="force", display=True)
+    assert glob.glob("force.html")
 
 
 def test_heatmap_plot():
@@ -636,5 +646,5 @@ def test_waterfall_plot():
     """Assert that the waterfall_plot method work as intended."""
     atom = ATOMClassifier(X_class, y_class, random_state=1)
     pytest.raises(NotFittedError, atom.waterfall_plot)
-    atom.run("LR", metric="f1_macro")
+    atom.run("Tree", metric="f1_macro")
     atom.waterfall_plot(display=False)
